#!/usr/bin/perl

# $Id$

use File::Compare;

sub usage{
  print "usage: difftree.pl <ref_dir> <target_dir>\n";
  exit;
}

if (scalar(@ARGV) != 2){
  print scalar(@ARGV), "\n";
  &usage;
  exit;
}

$REF_DIR = shift(@ARGV);
$TARGET_DIR = shift(@ARGV);

open(PIPE, "find $REF_DIR -print | ");
while(<PIPE>){
  $file = $_;
  chomp($file);
  $ref_dir{$file} = 1;
#  print "REF:$file\n";
}
close(PIPE);

open(PIPE, "find $TARGET_DIR -print | ");
while(<PIPE>){
  $file = $_;
  chomp($file);
  $target_dir{$file} = 1;
#  print "TARGET:$file\n";
}
close(PIPE);

foreach $file (sort keys %ref_dir){
  $file_base = &base_file($file);
  $target_file = $TARGET_DIR . $file_base;

  next if ($file =~ /\/RCS\//);

#print "\$file=$file\n\$target_file=$target_file\n";
#print "\$file_base=$file_base\n";

  if (($ref_dir{$file} == 1) and ($target_dir{$target_file} != 1)){
#    print "\$ref_dir{$file}=$ref_dir{$file}\n";
#    print "\$target_dir{$target_file}=$target_dir{$target_file}\n";
    print "$file_base: DELETED\n";
    push(@deleted, $file_base);
  } else {
    ($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size,
     $atime, $mtime, $ctime, $blksize, $blocks) = stat($file);
    $ref_size = $size;
    $ref_mtime = $mtime;

    ($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size,
     $atime, $mtime, $ctime, $blksize, $blocks) = stat($target_file);
    $target_size = $size;
    $target_mtime = $mtime;

    if ($ref_size != $target_size){
      print "$file_base: DIFF SIZE\n";
      print "\$target_size=$target_size, \$ref_size=$ref_size\n";
      push(@different, $file_base);
    } else {
      if (compare($file, $target_file)){
	print "$file_base: DIFFERENT\n";
      }

    }

  }

  $ref_dir{$file} = 2;
}

foreach $file (keys %target_dir){
#print "$file\n";
  $file_base = &base_file($file);
  $ref_file = $REF_DIR . $file_base;
  $target_file = $TARGET_DIR . $file_base;

  next if ($ref_dir{$ref_file} == 2);  # checked
#print "\$ref_dir{$ref_file}=$ref_dir{$ref_file}\n";
#print "\$target_dir{$target_file}=$target_dir{$target_file}\n";

  if ($target_dir{$target_file} == 1){
    print "$file_base: NEW\n";
    push(@new, $file_base);
  }
}


sub base_file{
  my $filename = shift;

  if ($filename =~ /^$REF_DIR/){
    $filename =~ s/^$REF_DIR//;
  } elsif ($filename =~ /^$TARGET_DIR/){
    $filename =~ s/^$TARGET_DIR//;
  }
  return $filename;

}
